from .fcn import (
    FCN8s,
    FCN16s,
    FCN32s,
    VggFCN8s,
    VggFCN16s,
    VggFCN32s,
    VggFCNs,
    VggFCN,
    ResFCN,
    DenseFCN,
    MobileFCN,
    MobileFCN1
)

from .unet import UNet
