"""
- author: Auorui (夏天是冰红茶)
- creation time:2022.10
- pyzjr is the Codebase accumulated by my python programming.
  At present, it is only for my personal use. If you want to
  use it, please contact me. Here are my email and WeChat.
- WeChat: zjricetea
- Email: zjricetea@gmail.com / 2165648225@qq.com
- Note: Currently still being updated, please refer to the latest version for any changes that may occur

    https://www.pepy.tech/projects/pyzjr
"""
from pyzjr.version import pyzjr_info, __version__
from pyzjr.utils import *
from pyzjr.visualize import *
from pyzjr.data import *
from pyzjr.nn import *
from pyzjr.measure import *
from pyzjr.Z import *
from pyzjr.augmentation import *
